@@ -1,6 +1,6 @@
 partial.rfsrc <- function(
   object,
-  outcome.target = NULL,
+  m.target = NULL,
   partial.type = NULL,
   partial.xvar = NULL,
   partial.values = NULL,
@@ -64,7 +64,7 @@ partial.rfsrc <- function(
   object$yvar <- as.data.frame(object$yvar)
   colnames(object$yvar) <- yvar.names
   yfactor <- extract.factor(object$yvar)
-  outcome.target.idx <- get.outcome.target(family, yvar.names, outcome.target)
+  m.target.idx <- get.outcome.target(family, yvar.names, m.target)
   ## Get the y-outcome type and number of levels
   yvar.types <- get.yvar.type(family, yfactor$generic.types, yvar.names, object$coerce.factor)
   yvar.nlevels <- get.yvar.nlevels(family, yfactor$nlevels, yvar.names, object$yvar, object$coerce.factor)
@@ -104,22 +104,36 @@ partial.rfsrc <- function(
   terminal.quants.bits <- get.terminal.quants(terminal.quants, object$terminal.quants)
   seed <- get.seed(seed)
   do.trace <- get.trace(do.trace)
+  ## Check that htry is initialized.  If not, set it zero.
+  ## This is necessary for backwards compatibility with 2.3.0
+    if (is.null(object$htry)) {
+        htry <- 0
+    }
+    else {
+        htry <- object$htry
+    }
+    ## Marker for start of native forest topology.  This can change with the outputs requested.
+    ## For the arithmetic related to the pivot point, you need to refer to stackOutput.c and in
+    ## particular, stackForestOutputObjects().
+    pivot <- which(names(object$nativeArray) == "treeID")
   nativeOutput <- tryCatch({.Call("rfsrcPredict",
                                   as.integer(do.trace),
                                   as.integer(seed),
-                                  as.integer(oob.bits + bootstrap.bits + cr.bits), 
+                                  as.integer(
+                                      oob.bits +
+                                      bootstrap.bits +
+                                      cr.bits), 
                                   as.integer(
                                     samptype.bits +
                                       na.action.bits +
                                         terminal.qualts.bits +
                                           terminal.quants.bits +
                                             partial.bits),
+                                  ## >>>> start of maxi forest object >>>>
                                   as.integer(ntree),
                                   as.integer(n),
                                   as.integer(r.dim),
                                   as.character(yvar.types),
-                                  as.integer(outcome.target.idx),
-                                  as.integer(length(outcome.target.idx)),
                                   as.integer(yvar.nlevels),
                                   as.double(as.vector(yvar)),
                                   as.integer(ncol(xvar)),
@@ -131,47 +145,68 @@ partial.rfsrc <- function(
                                   as.double(object$case.wt),
                                   as.integer(length(event.info$time.interest)),
                                   as.double(event.info$time.interest),
+                                  as.integer(object$totalNodeCount),
+                                  as.integer(object$seed),
+                                  as.integer(htry),
                                   as.integer((object$nativeArray)$treeID),
                                   as.integer((object$nativeArray)$nodeID),
-                                  as.integer((object$nativeArray)$parmID),
+                                  list(as.integer((object$nativeArray)$parmID),
                                   as.double((object$nativeArray)$contPT),
                                   as.integer((object$nativeArray)$mwcpSZ),
-                                  as.integer(object$nativeFactorArray),
+                                  as.integer((object$nativeFactorArray)$mwcpPT)),
+                                  if (htry > 0) {
+                                      list(as.integer((object$nativeArray)$hcDim),
+                                      as.double((object$nativeArray)$contPTR))
+                                  } else { NULL },
+                                  if (htry > 1) {
+                                      lapply(0:htry-2, function(x) {as.integer(object$nativeArray[[pivot + 9 + (0 * htry) + x]])})
+                                  } else { NULL },
+                                  if (htry > 1) {
+                                      lapply(0:htry-2, function(x) {as.double(object$nativeArray[[pivot + 9 + (1 * htry) + x]])})
+                                  } else { NULL },
+                                  if (htry > 1) {
+                                      lapply(0:htry-2, function(x) {as.double(object$nativeArray[[pivot + 9 + (2 * htry) + x]])})
+                                  } else { NULL },
+                                  if (htry > 1) {
+                                      lapply(0:htry-2, function(x) {as.integer(object$nativeArray[[pivot + 9 + (3 * htry) + x]])})
+                                  } else { NULL },
+                                  if (htry > 1) {
+                                      lapply(0:htry-2, function(x) {as.integer(object$nativeArray[[pivot + 9 + (4 * htry) + x]])})
+                                  } else { NULL },
                                   as.integer(object$nativeArrayTNDS$tnRMBR),
                                   as.integer(object$nativeArrayTNDS$tnAMBR),
                                   as.integer(object$nativeArrayTNDS$tnRCNT),
                                   as.integer(object$nativeArrayTNDS$tnACNT),
-                                  as.integer(object$totalNodeCount),
-                                  as.integer(object$seed),
-                                  as.integer(get.rf.cores()),
-                                  ## Pruning disabled
-                                  as.integer(0),
-                                  ## Importance disabled
-                                  as.integer(0),
-                                  as.integer(NULL),
-                                  ## Subsetting disabled.
-                                  as.integer(0),
-                                  as.integer(NULL),
-                                    
-                                  as.integer(get.type(family, partial.type)),
-                                  as.integer(which(xvar.names == partial.xvar)),
-                                  as.integer(length(partial.values)),
-                                  as.double(partial.values),
-                                  as.integer(length(partial.xvar2)),
-                                  as.integer(match(partial.xvar2, xvar.names)),
-                                  as.double(partial.values2),
-                                  ## New data disabled.
-                                  as.integer(0),
-                                  as.integer(0),
-                                  as.double(NULL),
-                                  as.double(NULL),
                                   as.double((object$nativeArrayTNDS$tnSURV)),
                                   as.double((object$nativeArrayTNDS$tnMORT)),
                                   as.double((object$nativeArrayTNDS$tnNLSN)),
                                   as.double((object$nativeArrayTNDS$tnCSHZ)),
                                   as.double((object$nativeArrayTNDS$tnCIFN)),
                                   as.double((object$nativeArrayTNDS$tnREGR)),
-                                  as.integer((object$nativeArrayTNDS$tnCLAS)))}, error = function(e) {
+                                  as.integer((object$nativeArrayTNDS$tnCLAS)),
+                                  ## <<<< end of maxi forest object <<<<
+                                  as.integer(m.target.idx),
+                                  as.integer(length(m.target.idx)),
+                                  as.integer(0),  ## Pruning disabled
+                                    
+                                  as.integer(0),     ## Importance disabled
+                                  as.integer(NULL),  ## Importance disabled
+                                  ## Partial variables enabled.  Note the as.integer is needed.
+                                  list(as.integer(get.type(family, partial.type)),
+                                       as.integer(which(xvar.names == partial.xvar)),
+                                       as.integer(length(partial.values)),
+                                       as.double(partial.values),
+                                       as.integer(length(partial.xvar2)),
+                                       if (length(partial.xvar2) == 0) NULL else as.integer(match(partial.xvar2, xvar.names)),
+                                       as.double(partial.values2)),
+                                  as.integer(0),     ## Subsetting disabled.
+                                  as.integer(NULL),  ## Subsetting disabled.
+                                  as.integer(0),    ## New data disabled.
+                                  as.integer(0),    ## New data disabled.
+                                  as.double(NULL),  ## New data disabled.
+                                  as.double(NULL),  ## New data disabled.
+                                  as.integer(ntree), ## err.block is hard-coded.
+                                  as.integer(get.rf.cores()))}, error = function(e) {
                                     print(e)
                                     NULL})
   ## check for error return condition in the native code
@@ -337,8 +372,8 @@ partial.rfsrc <- function(
         iter.end   <- 0
         offset <- vector("list", class.count)
         for (p in 1:length(partial.values)) {
-          for (k in 1:length(outcome.target.idx)) {
-            target.idx <- which (class.index == outcome.target.idx[k])
+          for (k in 1:length(m.target.idx)) {
+            target.idx <- which (class.index == m.target.idx[k])
             if (length(target.idx) > 0) {
               iter.start <- iter.end
               iter.end <- iter.start + ((1 + levels.count[target.idx]) * n)
@@ -346,8 +381,8 @@ partial.rfsrc <- function(
             }
           }
         }
-        for (i in 1:length(outcome.target.idx)) {
-          target.idx <- which (class.index == outcome.target.idx[i])
+        for (i in 1:length(m.target.idx)) {
+          target.idx <- which (class.index == m.target.idx[i])
           if (length(target.idx) > 0) {
             ens.names <- list(NULL, c("all", levels.names[[target.idx]]), NULL)
             ## Incoming from the native code:
@@ -373,8 +408,8 @@ partial.rfsrc <- function(
         iter.end   <- 0
         offset <- vector("list", regr.count)
         for (p in 1:length(partial.values)) {
-          for (k in 1:length(outcome.target.idx)) {
-            target.idx <- which (regr.index == outcome.target.idx[k])
+          for (k in 1:length(m.target.idx)) {
+            target.idx <- which (regr.index == m.target.idx[k])
             if (length(target.idx) > 0) {
               iter.start <- iter.end
               iter.end <- iter.start + n
@@ -382,8 +417,8 @@ partial.rfsrc <- function(
             }
           }
         }
-        for (i in 1:length(outcome.target.idx)) {
-          target.idx <- which (regr.index == outcome.target.idx[i])
+        for (i in 1:length(m.target.idx)) {
+          target.idx <- which (regr.index == m.target.idx[i])
           if (length(target.idx) > 0) {
             ens.names <- list(NULL, NULL)
             ## Incoming from the native code:
