@@ -1,8 +1,9 @@
 predict.rfsrc <-
   function(object,
            newdata,
-           outcome.target=NULL,
+           m.target=NULL,
            importance = c(FALSE, TRUE, "none", "permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble")[1],
+           err.block = NULL,
            na.action = c("na.omit", "na.impute"),
            outcome = c("train", "test"),
            proximity = FALSE,
@@ -19,12 +20,13 @@ predict.rfsrc <-
 {
   result.predict <- generic.predict.rfsrc(object,
                                           newdata,
-                                          outcome.target = outcome.target,
+                                          m.target = m.target,
                                           importance = importance,
+                                          err.block = err.block,
                                           na.action = na.action,
                                           outcome = outcome,
                                           proximity = proximity,
-                                          forest.wt = forest.wt,                                          
+                                          forest.wt = forest.wt,
                                           ptn.count = ptn.count,
                                            
                                           var.used = var.used,
